# coding: utf-8
##############################################################################
#
#----------------------------------------------------------------------------------------------------------------------------------------
#   mhpatch.py
#
#   Collection of patch and utility methods.
#
#   Copyright (c) 2004 Zope Japan KK.(Atsushi Shibata)
#   All Rights Reserved.
#----------------------------------------------------------------------------------------------------------------------------------------
#
# Copyright (c) 2004 Zope Japan KK.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__doc__="""mhpatch.py : Initialize functions for ZJMailHost'
$Id: mhpatch.py,v 1.4 2004/12/24 06:43:45 ats_shib Exp $"""

__version__='$Revision: 1.4 $'[11:-2]
__product_version__ = "ZJMailHost 0.8a"

from cStringIO import StringIO
import types
from email.Header import Header
import rfc822

from DateTime import DateTime
from Products.MailHost.MailHost import MailHostError,_encode


def patched_manage_makeChanges(self,title,smtp_host,smtp_port,\
                                REQUEST=None,fromcharset=None):
    'make the changes,additinal fromcharset parameter'

    title=str(title)
    smtp_host=str(smtp_host)
    smtp_port=int(smtp_port)
    fromc_harset=str(fromcharset)
    if fromc_harset == 'autodetect':
        fromc_harset = None

    self.title=title
    self.smtp_host=smtp_host
    self.smtp_port=smtp_port
    self.fromcharset = fromc_harset
    if REQUEST is not None:
        msg = 'MailHost updated'
        return self.manage_main( self
                               , REQUEST
                               , manage_tabs_message=msg
                               )


def patched_send(self, messageText, mto=None, mfrom=None, subject=None,
                 encode=None,charset=None,fromcharset=None):
    #Patched method for MailHost.send
    #Try to convert encode of message body and send it

    if not charset:
        try:
            charset = self.charset
        except:
            charset = 'iso-2022-jp'
    if not fromcharset:
        try:
            fromcharset = self.fromcharset
        except:
            fromcharset = None

    if cs_converter.has_key(charset):
        messageText = cs_converter[charset](messageText,fromcharset)
        subject = cs_converter[charset](subject,fromcharset)
        mto = cs_converter[charset](mto,fromcharset)
        mfrom = cs_converter[charset](mfrom,fromcharset)

    messageText, mto, mfrom = \
            _patched_mungeHeaders( messageText, mto, mfrom, subject)
    messageText = _encode(messageText, encode)
    self._send(mfrom, mto, messageText)

def _patched_mungeHeaders( messageText, mto=None, mfrom=None,
                            subject=None,charset='iso-2022-jp'):
    """Sets missing message headers, encode header, and deletes Bcc.
       returns fixed message, fixed mto and fixed mfrom"""
    mfile=StringIO(messageText.lstrip())
    mo=rfc822.Message(mfile)

    # Parameters given will *always* override headers in the messageText.
    # This is so that you can't override or add to subscribers by adding them to
    # the message text.
    if subject:
        mo['Subject'] = subject
    elif not mo.getheader('Subject'):
        mo['Subject'] = '[No Subject]'

    if mto:
        if isinstance(mto, types.StringType):
            mto = [rfc822.dump_address_pair(addr)\
                            for addr in rfc822.AddressList(mto) ]
        if not mo.getheader('To'):
            mo['To'] = ','.join(mto)
    else:
        mto = []
        for header in ('To', 'Cc', 'Bcc'):
            v = mo.getheader(header)
            if v:
                mto += [rfc822.dump_address_pair(addr)\
                            for addr in rfc822.AddressList(v)]
        if not mto:
            raise MailHostError, "No message recipients designated"

    if mfrom:
        mo['From'] = mfrom
    else:
        if mo.getheader('From') is None:
            raise MailHostError,"Message missing SMTP Header 'From'"
        mfrom = mo['From']

    if mo.getheader('Bcc'):
        mo.__delitem__('Bcc')

    if not mo.getheader('Date'):
        mo['Date'] = DateTime().rfc822()

    #Add header,mime-version,content-type
    if not mo.getheader('Mime-Version'):
        mo['Mime-Version']='1.0'
    if not mo.getheader('Content-Type'):
        mo['Content-Type']='text/plain; charset='+charset

    #
    # Encode header
    #
    if encoders.has_key(charset):
        for header in ('To','From','Cc','Subject'):
            if mo.getheader(header):
                mo[header] = encoders[charset](mo[header])
    
        mfrom =  encoders[charset](mfrom)
        mto = [encoders[charset](s) for s in mto]

    mo.rewindbody()
    finalmessage = mo
    finalmessage = mo.__str__() + '\n' + mfile.read()
    mfile.close()
    return finalmessage, mto, mfrom

#Encoders

#Escape code for iso-2022-jp(JIS)
esc_code = chr(033)

def _iso_2022_jp_encoder(hdrstr):
    """iso-2022-jp(JIS) Encoder
       Require JapaneseCodecs / CJKCodecs"""
    esc_blocks = hdrstr.split(esc_code)
    rstr = ""

    for block in esc_blocks:
        if block[:2] in ('$@','$B','(D','(I'):
            rstr += str(Header(esc_code+block,'iso-2022-jp'))
        elif block[:2] in ('(B','(J'):
            rstr += block[2:]
        else:
            rstr += block
    return rstr

encoders = {'iso-2022-jp':_iso_2022_jp_encoder,}

#Converters

#iso-2022-jp converter
def _iso_2022_jp_converter(msgstr,fromcode=None):
    """iso-2022-jp(JIS) Converter
       Require pykf module"""
    try:
        if isinstance(msgstr, types.UnicodeType):
            #Try to change encode using codecs
            msgstr = msgstr.encode('iso-2022-jp')
        else:
            import pykf
            codemap = {'euc-jp'     :pykf.EUC,
                       'utf-8'      :pykf.UTF8,
                       'shift_jis'  :pykf.SJIS,}
                       
            pkfromcode = pykf.UNKNOWN
            #Detect charcode using pykf.guess
            if not fromcode:
                pkfromcode = pykf.guess(msgstr)
            elif fromcode in codemap.keys():
                pkfromcode = codemap[fromcode]
            if pkfromcode == pykf.UTF8:
                msgstr = unicode(msgstr,'utf-8','replace').encode('iso-2022-jp')
            else:
                msgstr = pykf.tojis(msgstr,pkfromcode)

    except ImportError,e:
        pass
    
    return msgstr

cs_converter = {'iso-2022-jp':_iso_2022_jp_converter,}

if __name__ == '__main__':
    #Test _patched_mungeHeaders
    body = """From: "柴田" <shibata2@zope.co.jp>
To: "しばた" <shibata@zope.co.jp>,"新発田" <shibata@zope.co.jp>
Subject: ABCD件名DEF 長い件名　短い件名　MIMEヘッダーEnコーディング

このメールはテスト用のメールです。このメールはテスト用のメールです。
このメールはテスト用のメールです。このメールはテスト用のメールです。
"""
    body = unicode(body,'utf-8').encode('iso-2022-jp')

    mbody,tostr,fromstr = _patched_mungeHeaders(body,'shibata@zope.co.jp')
    print tostr,fromstr
    print "-"*40
    print mbody

    #Test charset converter
    ustr = '変換テスト用の文章'
    sjstr = unicode(ustr,'utf-8').encode('shift_jis')
    eucstr = unicode(ustr,'utf-8').encode('euc-jp')

    print "utf-8"
    print _iso_2022_jp_converter(ustr)
    print _iso_2022_jp_converter(ustr,'utf-8')

    print "-"*40
    print "shift_jis"
    print _iso_2022_jp_converter(sjstr)
    print _iso_2022_jp_converter(sjstr,'shift_jis')

    print "-"*40
    print "euc-jp"
    print _iso_2022_jp_converter(eucstr)
    print _iso_2022_jp_converter(eucstr,'euc-jp')

