##############################################################################
#
#----------------------------------------------------------------------------------------------------------------------------------------
#   ZMILocalizer
#
#   Product to localize/internationalize Zope Management Interface.
#
#   Copyright (c) 2004 Zope Japan KK.(Atsushi Shibata)
#   All Rights Reserved.
#----------------------------------------------------------------------------------------------------------------------------------------
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

#import from Zope core
from Globals import DTMLFile,HTMLFile
from App.FindHomes import INSTANCE_HOME, SOFTWARE_HOME, ZOPE_HOME
from OFS import ObjectManager
from OFS.Application import get_products
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from zLOG import LOG,ERROR,PROBLEM

from AccessControl import ModuleSecurityInfo, allow_module
from AccessControl.Permissions import view

#import from local moudles
from patchcatalog import build,read_patch_file

#import from python modules
import os,sys

#Initialize functions for ZMI Localizer

__doc__="""__init__.py : Initialize functions for ZMI Localizer'
$Id: __init__.py,v 1.7 2004/12/24 08:13:14 ats_shib Exp $"""

__version__='$Revision: 1.7 $'[11:-2]
__product_version__ = "ZMI Localizer 0.9a"

security = ModuleSecurityInfo('Products.ZMILocalizer')
allow_module('Products.ZMILocalizer')
allow_module('Products.ZMILocalizer.utilities')

security.declareProtected(view, 'get_patch_list')
def get_patch_list():
    """Returns patch list"""
    return patch_list

security.declareProtected(view, 'get_tabnames')
def get_tabnames():
    """Return tab names"""
    return tabnames

def patch_submit_value(request,prefix='SUBMIT'):
    """Patch for SUBMIT value"""
    retv = None
    for key in request.form.keys():
        if key.find(prefix+'_') == 0:
            retv = key[len(prefix)+1:]
            break
    return retv

def dtm_manage_edit(self,data,title,SUBMIT='Change',dtpref_cols='100%',
                         dtpref_rows='20',REQUEST=None):
    """Patch method for DTMLMethod manage_edit"""
    if REQUEST:
        p_submit = patch_submit_value(REQUEST)
        if p_submit:
            SUBMIT=p_submit
    return orig_dtm_manage_edit(self,data,title,SUBMIT,\
                                    dtpref_cols,dtpref_rows,REQUEST)

def dtd_manage_edit(self,data,title,SUBMIT='Change',dtpref_cols='100%',
                         dtpref_rows='20',REQUEST=None):
    """Patch method for DTMLMethod manage_edit"""
    if REQUEST:
        p_submit = patch_submit_value(REQUEST)
        if p_submit:
            SUBMIT=p_submit
    return orig_dtd_manage_edit(self,data,title,SUBMIT,\
                                    dtpref_cols,dtpref_rows,REQUEST)

def acc_manage_users(self,submit=None,REQUEST=None,RESPONSE=None):
    """Patch method for DTMLMethod manage_edit"""
    if REQUEST:
        p_submit = patch_submit_value(REQUEST,'submit')
        if p_submit:
            submit=p_submit
    return orig_acc_manage_users(self,submit,REQUEST,RESPONSE)

def da_manage_testForm(self, REQUEST):
    """Patch method for Shared.DC.ZRDB.DA.manage_testForm"""
    import DocumentTemplate
    import string
    from Shared.DC.ZRDB.Aqueduct import nicify
    id = self.title_or_id(),
    tabs = '<dtml-var manage_tabs>'
    arguments = self._arg
    action = 'manage_test'
    id = ''
    items=arguments.items()
    input_src = ("%s\n%s%s" % (
        '<dtml-var manage_page_header>\n'
        '<!-- %s -->'
        '<body bgcolor="#FFFFFF" link="#000099" vlink="#555555">\n%s\n'
        '<form action="&dtml-URL2;/&dtml-id;/%s" '
        'method="get">\n'
        '<h2>%s <dtml-translate domain=zmi>Input Data</dtml-translate></h2>\n'
        '<dtml-translate domain=zmi>Enter query parameters:</dtml-translate><br>'
        '<table>\n'
        % (id, tabs, action,id),
        string.joinfields(
            map(
                lambda a:
                ('<tr> <th>%s</th>\n'
                 '     <td><input name="%s"\n'
                 '                width=30 value="%s">'
                 '     </td></tr>'
                 % (nicify(a[0]),
                    (
                        a[1].has_key('type') and
                        ("%s:%s" % (a[0],a[1]['type'])) or
                        a[0]
                        ),
                    a[1].has_key('default') and a[1]['default'] or ''
                    ))
                , items
                ),
        '\n'),
        '\n<tr><td colspan=2 align=center>\n'
        '<input type="SUBMIT" name="SUBMIT" value="<dtml-translate domain=zmi>Submit Query</dtml-translate>">\n'
        '<dtml-if HTTP_REFERER>\n'
        '  <input type="SUBMIT" name="SUBMIT" value="Cancel">\n'
        '  <INPUT NAME="CANCEL_ACTION" TYPE="HIDDEN"\n'
        '         VALUE="&dtml-HTTP_REFERER;">\n'
        '</dtml-if>\n'
        '</td></tr>\n</table>\n</form>\n</body>\n</html>\n'
        )
    )
    return DocumentTemplate.HTML(input_src)(self, REQUEST, HTTP_REFERER='')

def da_manage_test(self, REQUEST):
    """Patch method for Shared.DC.ZRDB.DA.manage_test"""
    import sys
    import DocumentTemplate
    from DocumentTemplate.html_quote import html_quote
    """Patch for Test an SQL method."""
    try:    src=self(REQUEST, src__=1)
    except: src="<dtml-translate domain=zmi>Could not render the query template!</dtml-translate>"
    result=()
    t=v=tb=None
    try:
        try:
            src, result=self(REQUEST, test__=1)
            if find(src,'\0'):
                src=html_quote(join(split(src,'\0'),'\n'+'-'*60+'\n'))
            if result._searchable_result_columns():
                r=custom_default_report(self.id, result)
            else:
                r='<dtml-translate domain=zmi>This statement returned no results.</dtml-translate>'
        except:
            t, v, tb = sys.exc_info()
            r='<strong><dtml-translate domain=zmi>Error, </dtml-translate><em>%s</em>:</strong> %s' % (t, v)

        report=DocumentTemplate.HTML(
            '<dtml-var manage_page_header>\n'
            '<dtml-var manage_tabs>\n<hr>\n%s\n\n'
            '<hr><strong><dtml-translate domain=zmi>SQL used:</dtml-translate></strong><br>\n<pre>\n%s\n</pre>\n<hr>\n'
            '</body></html>'
            % (r, src))

        report=apply(report,(self,REQUEST),{self.id:result})

        if tb is not None:
            self.raise_standardErrorMessage(
                None, REQUEST, t, v, tb, None, report)

        return report

    finally: tb=None

def patched_publish(request, module_name, after_list, debug=0):
    """Patch method for ZPublisher.Publish.publish
    Replace SUBMIT value, 'SUBMIT_xxx' with 'SUBMIT' : 'xxx'"""
    return orig_publish(request, module_name, after_list, debug=debug)

replacekey4submit = '/replace2/'

def patched_processInputs(self):
    """Patch method for ZPublisher.HTTPRequest.processInputs
    Replace SUBMIT value, 'SUBMIT/name2value/xxx' with 'SUBMIT' : 'xxx'"""
    r = orig_processInputs(self)
    for key in self.form.keys():
        l = key.find(replacekey4submit)
        if l >= 0:
            #Replace value
            #Delete old value
            del self.form[key]
            #Append new value
            self.form[key[:l]] = key[l+len(replacekey4submit):]
    return r


def initialize(context):

    try:
        #Try to import PTS
        from Products.PlacelessTranslationService\
            import PlacelessTranslationService

        from utilities import patched_read_raw,_patched_cook_check,\
                              build_patches,label_patcher,message_patcher
        #patch for static templates
        HTMLFile.read_raw = patched_read_raw
        PageTemplateFile._cook_check = _patched_cook_check

        #Re-load files
        #For External Editor,or another monkey-patcher :-)
        ObjectManager.ObjectManager.manage_main.read_raw(reload=True)

        #Patch for edit methods
        from OFS.DTMLMethod import DTMLMethod
        from OFS.DTMLDocument import DTMLDocument
    
        global orig_dtm_manage_edit,orig_dtd_manage_edit
    
        orig_dtm_manage_edit = DTMLMethod.manage_edit
        DTMLMethod.manage_edit = dtm_manage_edit
    
        #orig_dtd_manage_edit = DTMLDocument.manage_edit
        #DTMLDocument.manage_edit = dtd_manage_edit
    
        from AccessControl.User import UserFolder
    
        global orig_acc_manage_users
    
        orig_acc_manage_users = UserFolder.manage_users
        UserFolder.manage_users = acc_manage_users
    
        global original_default_input_form
        from Shared.DC.ZRDB.DA import DA
    
        DA.manage_testForm = da_manage_testForm
        DA.manage_test = da_manage_test
    
        global patch_list,tabnames
    
        patch_list = None   #list of patch objects
        tabnames = None     #list for tablenames

        #Add patch methods to special namespace('_').
        from DocumentTemplate.cDocumentTemplate import TemplateDict
        TemplateDict.__dict__['translate_tab'] = label_patcher
        TemplateDict.__dict__['translate_message'] = message_patcher

        #build patch objects
        patch_list,tabnames = build_patches(context)

        #Patch for ZPublisher.Publish, to patch SUBMIT values
        from ZPublisher import Publish

        #Patch ZPublisher.HTTPRequest.processInputs
        global orig_processInputs

        from ZPublisher.HTTPRequest import HTTPRequest
        orig_processInputs = HTTPRequest.processInputs
        HTTPRequest.processInputs = patched_processInputs
    
        #Read patch list from product directory
        for prd in get_products():
            # prd is a tupple,(priority, dir_name, index, base_dir)
            p_file = read_patch_file(os.path.join(prd[3], prd[1]))
            if p_file:
                patch_list += p_file
    except ImportError:
        exc=sys.exc_info()
        LOG('ZMI Localizer', ERROR,
            ":ZMI Localizer requires 'Placeless Translation Services'")
    except:
        exc=sys.exc_info()
        LOG('ZMILocalizer', ERROR, ':Some error occured','', exc)
