##############################################################################
#
#----------------------------------------------------------------------------------------------------------------------------------------
#   utilities.py
#
#   Utility methods.
#
#   Copyright (c) 2004 Zope Japan KK.(Atsushi Shibata)
#   All Rights Reserved.
#----------------------------------------------------------------------------------------------------------------------------------------
#
# Copyright (c) 2004 Zope Japan KK.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

#import from Zope core
from Products.ZMILocalizer import get_patch_list,get_tabnames
from Products.PlacelessTranslationService import getTranslationService
from Products.PageTemplates.PageTemplateFile import PageTemplateFile,\
                            DevelopmentMode,XML_PREFIX_MAX_LENGTH,sniff_type
from zLOG import LOG,ERROR

#import from local moudles
from elementpatcher import tab_domain,zmi_domain,elementpatcher
from patchcatalog import build

#import from python modules
import os
import re
from types import ListType

innerpath = '/'

__doc__="""utilities.py :utility,patch functions'
$Id: utilities.py,v 1.6 2004/12/24 08:13:14 ats_shib Exp $"""

__version__='$Revision: 1.6 $'[11:-2]

def get_relative_path(path):
    """Return relative path, compared with ZOPE_HOME / INSTANCE_HOME"""
    rpath = path
    #Manage pathname
    if path.find(ZOPE_HOME) == 0:
        rpath = path[len(ZOPE_HOME):]
    elif path.find(INSTANCE_HOME) == 0:
        rpath = path[len(INSTANCE_HOME):]
    if os.path.sep != innerpath:
        rpath = rpath.replace(os.path.sep,innerpath)
    return rpath

def replace_elements(src,path,isZPT=False):
    """Replace string in ZMI - DTMLFile / ZPTFile"""
    r_path = get_relative_path(path)
    #make elements list for path
    elms = []
    #append for descriptions
    for patcher in get_patch_list():
        elms += patcher.get_matched(r_path)

    #Go replace!
    pt = elementpatcher(src,elms)
    pt.replace()
    s = pt.render(isZPT)
    return s

#
# Patch methods
#

#patch for dtml
def patched_read_raw(self,reload=False):
    """Patch method for DTMLFile.read_raw
       Replace elements - buttons,form labels,descriptions in ZMI"""
    #Set flag to reload source
    if reload:
        self.edited_source = ''
        return

    if self.edited_source: return self.edited_source
    if not os.path.exists(self.raw):
        #!!! Must be replaced to use 'LOG' or something...
        raise OSError('file not found: %s' % self.raw)

    if self.raw:
        src = open(self.raw,'r').read()
        src = replace_elements(src,self.raw)
        return src

    return ''

#patch for ZPT
def _patched_cook_check(self):
    """Patch for PageTemplateFile._cook_check"""
    if self._v_last_read and not DevelopmentMode:
        return
    __traceback_info__ = self.filename
    try:
        mtime = os.path.getmtime(self.filename)
    except OSError:
        mtime = 0
    if self._v_program is not None and mtime == self._v_last_read:
        return
    f = open(self.filename, "rb")
    try:
        text = f.read(XML_PREFIX_MAX_LENGTH)
    except:
        f.close()
        raise
    t = sniff_type(text)
    if t != "text/xml":
        # For HTML, we really want the file read in text mode:
        f.close()
        f = open(self.filename)
        text = ''
    text += f.read()
    f.close()

    #patch PageTemplate
    text = replace_elements(text,self.filename,True)

    self.pt_edit(text, t)
    self._cook()
    if self._v_errors:
        LOG('PageTemplateFile', ERROR, 'Error in template',
            '\n'.join(self._v_errors))
        return
    self._v_last_read = mtime


#patch for tab
def label_patcher(self,doc,s,REQUEST=None):
    """Patch ZMI Tab using PlacelessTranslationServices.translate"""
    try:
        #try to reset PTS instance
        from Products import PlacelessTranslationService
        root = doc.getPhysicalRoot()
        PlacelessTranslationService.translation_service =\
            root.unrestrictedTraverse(('', 'Control_Panel',\
                            PlacelessTranslationService.cp_id),None)
        trns = getTranslationService().translate(domain=tab_domain,\
                                        msgid=s,context=REQUEST)
        if trns:
            s = trns
    except:
        pass
    return s


#patch for tab
def message_patcher(self,s,REQUEST=None):
    """Patch message on ZMI Tab using PlacelessTranslationServices.translate"""
    try:
        #try to reset PTS instance
        from Products import PlacelessTranslationService
        trns = getTranslationService().translate(domain=zmi_domain,\
                                        msgid=s,context=REQUEST)
        if trns:
            s = trns
    except:
        pass
    return s


def patch_product_tab(cls):
    """Make a patch to Product's tab lebal"""
    ret_list = []
    try:
        for d in cls.manage_options:
            #Loop around Product's manage_options
            if d.has_key('label'):
                orig_label = d['label']
                if type(orig_label) != ListType:
                    d.update({'label':[label_patcher,orig_label]})
                    ret_list.append(orig_label)
    except:
        pass
    return ret_list


def build_patches(context):
    #Read patch catalog
    pl = list(build())

    #Patch tabs

    #Get ControlPanel
    #cp = context._ProductContext__app.Control_Panel
    tn = []
    #Patch for tabs in existing products
    #for d in cp.all_meta_types():
    #    if d.has_key('instance'):
    #        tn += patch_product_tab(d['instance'])

    #Patch for Control Panels and classes under it.
    #tn += patch_product_tab(cp)

    #for cls in cp.objectValues():
    #    tn += patch_product_tab(cls)

    #Patch for rest classes.
    #from App.ApplicationManager import DatabaseManager
    #tn += patch_product_tab(DatabaseManager)

    return pl,tn
